from flask import Flask, request
from flask import jsonify
from doi import doi_parser
from zbl import zbl_parser
from mr import mr_parser
from all import all_parser
from shared import json_parser
from shared import xml_parser
from shared import txt_parser
from mr import mr_advanced_parser
from mr import mr_advanced_finder
from config.swagger import swagger_config 
from flasgger import Swagger, swag_from
from datetime import timedelta
import requests_cache


app = Flask(__name__)
app.config["SWAGGER"] = {"title": "Swagger-UI", "uiversion": 3,'openapi': '3.0.3'}
swagger = Swagger(app, config=swagger_config)

requests_cache.install_cache(
    cache_name='requests_cache', expire_after=timedelta(days=14))

def response_ok_with_jsonify(list_output):
    response = jsonify(list_output)
    response.status_code = 200
    return response


def response_unsupported_format():
    response = []
    response.status_code = 415
    return response

def response_semantic_error():
    response = []
    response.status_code = 422
    return response

def parse_request(parser):
    try:
        if request.content_type == "application/xml":
            return response_ok_with_jsonify(xml_parser.parse_xml(request.data, parser))
        if request.content_type == "text/plain":
            return response_ok_with_jsonify(txt_parser.parse_txt(request.data, parser))
        if request.content_type == "application/json":
            return response_ok_with_jsonify(json_parser.parse_json(request.data, parser))
        return response_unsupported_format()
    except:
        return response_semantic_error()



@app.route("/doi", methods=["POST"])
@swag_from("./docs/doi_config.yml")
def doi_parse():
    return parse_request(doi_parser.parser)


@app.route("/zbl", methods=["POST"])
@swag_from("./docs/zbl_config.yml")
def zbl_parse():
    return parse_request(zbl_parser.parser)


@app.route("/mr", methods=["POST"])
@swag_from("./docs/mr_config.yml")
def mr_parse():
    if mr_advanced_finder.check_access_to_advanced_search():
        return parse_request(mr_advanced_parser.parser)
    return parse_request(mr_parser.parser)


@app.route("/all", methods=["POST"])
@swag_from("./docs/all_config.yml")
def all_parse():
    return parse_request(all_parser.parser)


if __name__ == '__main__':
    app.run(debug=True)